@@ -6,6 +6,23 @@
 #include "accountmanager.h"
 #include <QDebug>
 #include <QUrlQuery>
+#include <KLocalizedString>
+
+Relationship* Identity::relationship() const {
+    return m_relationship;
+}
+
+void Identity::setRelationship(Relationship *r) {
+    if (m_relationship == r) {
+        return;
+    }
+    if (m_relationship != nullptr) {
+        // delete old relationship object if we receive a new one
+        delete m_relationship;
+    }
+    m_relationship = r;
+    Q_EMIT relationshipChanged();
+}
 
 Account::Account(const QString &name, const QString &instance_uri, QObject *parent)
     : QObject(parent)
@@ -49,13 +66,12 @@ void Account::registerApplication()
 
     // register
     const QUrl regUrl = apiUrl("/api/v1/apps");
-    QJsonObject obj;
-
-    obj["client_name"] = "Tokodon";
-    obj["redirect_uris"] = "urn:ietf:wg:oauth:2.0:oob";
-    obj["scopes"] = "read write follow";
-    obj["website"] = "https://invent.kde.org/carlschwan/kmasto";
-
+    const QJsonObject obj = {
+        {"client_name", "Tokodon"},
+        {"redirect_uris", "urn:ietf:wg:oauth:2.0:oob"},
+        {"scopes", "read write follow"},
+        {"website", "https://invent.kde.org/network/tokodon"}
+    };
     const QJsonDocument doc(obj);
 
     post(regUrl, doc, false, [=](QNetworkReply *reply) {
@@ -388,14 +404,23 @@ void Account::buildFromSettings(const QSettings &settings)
     }
 }
 
-void Account::fetchAccount(int id, bool excludeReplies, std::function<void(QList<std::shared_ptr<Post>>)> final_cb)
+void Account::fetchAccount(qint64 id, bool excludeReplies, const QString &timelineName, const QString &fromId)
 {
     auto thread = std::make_shared<QList<std::shared_ptr<Post>>>();
+    // Fetch pinned posts if we are starting from the top
+    const bool fetchPinned = fromId.isNull();
 
     QUrl uriStatus(m_instance_uri);
     uriStatus.setPath(QString("/api/v1/accounts/%1/statuses").arg(id));
+    auto statusQuery = QUrlQuery();
     if (excludeReplies) {
-        uriStatus.setQuery(QUrlQuery{{"exclude_replies", "true"}});
+        statusQuery.addQueryItem("exclude_replies", "true");
+    }
+    if (!fetchPinned) {
+        statusQuery.addQueryItem("max_id", fromId);
+    }
+    if (!statusQuery.isEmpty()) {
+        uriStatus.setQuery(statusQuery);
     }
 
     QUrl uriPinned(m_instance_uri);
@@ -420,7 +445,7 @@ void Account::fetchAccount(int id, bool excludeReplies, std::function<void(QList
             i++;
         }
 
-        final_cb(*thread);
+        Q_EMIT fetchedTimeline(timelineName, *thread);
     };
 
     auto onFetchAccount = [=](QNetworkReply *reply) {
@@ -439,7 +464,11 @@ void Account::fetchAccount(int id, bool excludeReplies, std::function<void(QList
             thread->push_back(p);
         }
 
-        get(uriPinned, true, onFetchPinned);
+        if (fetchPinned) {
+            get(uriPinned, true, onFetchPinned);
+        } else {
+            Q_EMIT fetchedTimeline(timelineName, *thread);
+        }
     };
 
     get(uriStatus, true, onFetchAccount);
@@ -778,9 +807,118 @@ Post *Account::newPost()
     return new Post(this);
 }
 
+void Account::executeAction(Identity *i, AccountAction accountAction, const QJsonObject &extraArguments)
+{
+    const QHash<AccountAction, QString> accountActionMap = {
+        {AccountAction::Follow, QStringLiteral("/follow")},
+        {AccountAction::Unfollow, QStringLiteral("/unfollow")},
+        {AccountAction::Block, QStringLiteral("/block")},
+        {AccountAction::Unblock, QStringLiteral("/unblock")},
+        {AccountAction::Mute, QStringLiteral("/mute")},
+        {AccountAction::Unmute, QStringLiteral("/unmute")},
+        {AccountAction::Feature, QStringLiteral("/pin")},
+        {AccountAction::Unfeature, QStringLiteral("/unpin")},
+        {AccountAction::Note, QStringLiteral("/note")},
+    };
+
+    const auto apiCall = accountActionMap[accountAction];
+
+    const auto account_id = QString::number(i->m_id);
+    const QString api_url = QStringLiteral("/api/v1/accounts/") + account_id + apiCall;
+    QUrl url = apiUrl(api_url);
+    const QJsonDocument doc(extraArguments);
+
+    post(url, doc, true, [=](QNetworkReply *reply) {
+        auto doc = QJsonDocument::fromJson(reply->readAll());
+        auto jsonObj = doc.object();
+
+        // Check if the request failed due to one account blocking the other
+        if (!jsonObj.value("error").isUndefined()) {
+            const QHash<AccountAction, QString> accountActionMap = {
+                {AccountAction::Follow, i18n("Could not follow account")},
+                {AccountAction::Unfollow, i18n("Could not unfollow account")},
+                {AccountAction::Block, i18n("Could not block account")},
+                {AccountAction::Unblock, i18n("Could not unblock account")},
+                {AccountAction::Mute, i18n("Could not mute account")},
+                {AccountAction::Unmute, i18n("Could not unmute account")},
+                {AccountAction::Feature, i18n("Could not feature account")},
+                {AccountAction::Unfeature, i18n("Could not unfeature account")},
+                {AccountAction::Note, i18n("Could not edit note about an account")},
+            };
+            const auto errorMessage = accountActionMap[accountAction];
+            Q_EMIT errorOccured(errorMessage);
+            return;
+        }
+        // If returned json obj is not an error, it's a relationship status.
+        // Returned relationship should have a value of true
+        // under either the "following" or "requested" keys.
+        auto relationship = i->relationship();
+        relationship->updateFromJson(jsonObj);
+
+        Q_EMIT i->relationshipChanged();
+    });
+}
+
+void Account::followAccount(Identity *identity, bool reblogs, bool notify)
+{
+    executeAction(identity, AccountAction::Follow, {
+        {"reblogs", reblogs},
+        {"notify", notify}
+    });
+}
+
+void Account::unfollowAccount(Identity *identity)
+{
+    executeAction(identity, AccountAction::Unfollow);
+}
+
+void Account::blockAccount(Identity *identity)
+{
+    executeAction(identity, AccountAction::Block);
+}
+
+void Account::unblockAccount(Identity *identity)
+{
+    executeAction(identity, AccountAction::Unblock);
+}
+
+void Account::muteAccount(Identity *identity, bool notifications, int duration)
+{
+    executeAction(identity, AccountAction::Mute, {
+        {"notifcations", notifications},
+        {"duration", duration}
+    });
+}
+
+void Account::unmuteAccount(Identity *identity)
+{
+    executeAction(identity, AccountAction::Unmute);
+}
+
+void Account::featureAccount(Identity *identity)
+{
+    executeAction(identity, AccountAction::Feature);
+}
+
+void Account::unfeatureAccount(Identity *identity)
+{
+    executeAction(identity, AccountAction::Unfeature);
+}
+
+void Account::addNote(Identity *identity, const QString &note)
+{
+    if (note.isEmpty()) {
+        executeAction(identity, AccountAction::Note);
+    } else {
+        executeAction(identity, AccountAction::Note, {
+            {"comment", note}
+        });
+    }
+}
+
 void Identity::fromSourceData(const QJsonObject &doc)
 {
-    m_id = doc["id"].toString().toInt();
+    m_id = doc["id"].toString().toULongLong();
     m_display_name = doc["display_name"].toString();
     m_acct = doc["acct"].toString();
     m_bio = doc["note"].toString();
@@ -807,3 +945,8 @@ void Identity::reparentIdentity(Account *parent)
 {
     m_parent = parent;
 }
+
+qint64 Identity::id() const
+{
+    return m_id;
+}
