@@ -22,6 +22,7 @@
 #include <QtNetwork/QNetworkRequest>
 
 #include "post.h"
+#include "relationship.h"
 
 class Account;
 
@@ -41,8 +42,12 @@ class Identity : public QObject
     Q_PROPERTY(int statusesCount MEMBER m_statusesCount CONSTANT)
     Q_PROPERTY(QJsonArray fields MEMBER m_fields CONSTANT)
 
+    Q_PROPERTY(Relationship *relationship READ relationship NOTIFY relationshipChanged)
+
 public:
-    int m_id;
+    qint64 id() const;
+
+    qint64 m_id;
     QString m_display_name;
     QString m_bio;
     QString m_acct;
@@ -60,6 +65,15 @@ public:
 
     QNetworkAccessManager *m_qnam;
     Account *m_parent;
+
+    Relationship* relationship() const;
+    void setRelationship(Relationship *r);
+
+Q_SIGNALS:
+    void relationshipChanged();
+
+private:
+    Relationship *m_relationship = nullptr;
 };
 
 class Account : public QObject
@@ -170,7 +184,7 @@ public:
     void fetchThread(const QString &postId, std::function<void(QList<std::shared_ptr<Post>>)> final_cb);
 
     /// Fetch account timeline
-    void fetchAccount(int id, bool excludeReplies, std::function<void(QList<std::shared_ptr<Post>>)> final_cb);
+    void fetchAccount(qint64 id, bool excludeReplies, const QString &timelineName, const QString& from_id = QString());
 
     // streaming
     QUrl streamingUrl(const QString &stream);
@@ -188,6 +202,47 @@ public:
         return m_allowedContentTypes;
     }
 
+    /// Follow the given account. Can also be used to update whether to show reblogs or enable notifications.
+    /// @param Identity identity The account to follow
+    /// @param bool reblogs Receive this account's reblogs in home timeline? Defaults to true.
+    /// @param bool notify Receive notifications when this account posts a status? Defaults to false.
+    Q_INVOKABLE void followAccount(Identity *identity, bool reblogs = true, bool notify = false);
+
+    /// Unfollow the given account.
+    /// @param Identity identity The account to unfollow
+    Q_INVOKABLE void unfollowAccount(Identity *identity);
+
+    /// Block the given account.
+    /// @param Identity identity The account to block
+    Q_INVOKABLE void blockAccount(Identity *identity);
+
+    /// Unblock the given account.
+    /// @param Identity identity The account to unblock
+    Q_INVOKABLE void unblockAccount(Identity *identity);
+
+    /// Mute the given account.
+    /// @param Identity identity The account to mute
+    /// @param bool notifications Whether notifications should also be muted, by default true
+    /// @param int duration How long the mute should last, in seconds. Defaults to 0 (indefinite).
+    Q_INVOKABLE void muteAccount(Identity *identity, bool notifications = true, int duration = 0);
+
+    /// Unmute the given account.
+    /// @param Identity identity The account to unmute
+    Q_INVOKABLE void unmuteAccount(Identity *identity);
+
+    /// Add the given account to the user's featured profiles.
+    /// @param Identity identity The account to feature
+    Q_INVOKABLE void featureAccount(Identity *identity);
+
+    /// Remove the given account from the user's featured profiles.
+    /// @param Identity identity The account to unfeature
+    Q_INVOKABLE void unfeatureAccount(Identity *identity);
+
+    /// Sets a private note on a user.
+    /// @param Identity identity The account to annotate
+    /// @param QString note The note to add to the account. Leave empty to remove the existing note.
+    Q_INVOKABLE void addNote(Identity *identity, const QString &note);
+
 Q_SIGNALS:
     void authenticated();
     void registered();
@@ -198,6 +253,8 @@ Q_SIGNALS:
     void fetchedInstanceMetadata();
     void invalidatedPost(Post *p);
     void notification(std::shared_ptr<Notification> n);
+    void followRequestBlocked();
+    void errorOccured(const QString &errorMessage);
 
 private:
     QString m_name;
@@ -218,6 +275,20 @@ private:
     QUrlQuery buildOAuthQuery() const;
     void mutatePost(std::shared_ptr<Post> p, const QString &verb, bool deliver_home = false);
 
+    enum AccountAction {
+        Follow,
+        Unfollow,
+        Block,
+        Unblock,
+        Mute,
+        Unmute,
+        Feature,
+        Unfeature,
+        Note
+    };
+
+    void executeAction(Identity *i, AccountAction accountAction, const QJsonObject &extraArguments = {});
+
     // updates and notifications
     void handleUpdate(const QJsonDocument &doc, const QString &target);
     void handleNotification(const QJsonDocument &doc);
