@@ -3,24 +3,30 @@
 
 #include "account.h"
 #include "timelinemodel.h"
+#include <qobjectdefs.h>
 
 class AccountModel : public TimelineModel
 {
     Q_OBJECT
     Q_PROPERTY(Identity *identity READ identity NOTIFY identityChanged)
+    Q_PROPERTY(Account *account READ account NOTIFY accountChanged)
+    Q_PROPERTY(bool isSelf READ isSelf NOTIFY identityChanged)
 
 public:
-    AccountModel(AccountManager *manager, int id, const QString &acct, QObject *parent = nullptr);
+    AccountModel(AccountManager *manager, qint64 id, const QString &acct, QObject *parent = nullptr);
 
     QString displayName() const override;
+    Account *account() const;
     Identity *identity() const;
+    bool isSelf() const;
+
     void fillTimeline(const QString &fromId = QString()) override;
-    bool canFetchMore(const QModelIndex &parent) const override;
 
 Q_SIGNALS:
     void identityChanged();
+    void accountChanged();
 
 private:
     std::shared_ptr<Identity> m_identity;
-    int m_id;
+    qint64 m_id;
 };
