@@ -98,8 +98,9 @@ int main(int argc, char *argv[])
     });
     qmlRegisterType<AttachmentEditorModel>("org.kde.kmasto", 1, 0, "AttachmentEditorModel");
     qRegisterMetaType<Account *>("Account*");
-    qRegisterMetaType<Post *>("Post*");
     qRegisterMetaType<Identity *>("Identity*");
+    qRegisterMetaType<Relationship *>("Relationship*");
+    qmlRegisterUncreatableType<Post>("org.kde.kmasto", 1, 0, "Post", "ENUM");
 
     QQmlApplicationEngine engine;
     engine.rootContext()->setContextObject(new KLocalizedContext(&engine));
